%%% Monte Carlo Dynamic Classifier tool - MCDC tool

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
% Floor, Boston, MA 02110-1301 USA.

%
% CheckGridTransformation.m
%

function CheckGridTransformation(f, gridData, varargin)

  aspect = GetAspect(varargin{:});

  range = GridRange(gridData);

  gridPoints = gridData.GetFlatMatrix();
  gridValues = f(gridPoints')';

  for i = 1:size(gridValues, 2)
    if ~IsValueInRange(gridValues(:, i), range)
      message = BuildLogMessage(gridPoints(:, i), gridValues(:, i));
      if aspect ~= 0
        aspect.Log(message);
      else
        disp(message);
      end
    end
  end

end

function result = GridRange(gridData)
  result = zeros(gridData.Dim, 2);
  for d = 1:gridData.Dim
    values = gridData.Grids{d};
    result(d, :) = [min(values) max(values)];
  end
end

function result = IsValueInRange(value, range)
  dim = size(value, 1);
  result = prod((range(1:dim, 1) <= value) .* (value <= range(1:dim, 2)));
end

function result = GetAspect(varargin)
  if size(varargin, 2) > 0
    result = varargin{1};
  else
    result = 0;
  end
end

function result = BuildLogMessage(point, value)
  result = sprintf('f([%s]) = ([%s]) is out of grids', ToCsv(point'), ToCsv(value'));
end

function result = ToCsv(values)
  result = sprintf('%g,', values);
  result = result(1:end-1);
end
