%%% Monte Carlo Dynamic Classifier tool - MCDC tool

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
% Floor, Boston, MA 02110-1301 USA.

%
% PlotGraph.m
% 
% plot による線形 2 次元グラフ
% 作成したグラフを PDF ファイルに保存することが可能
%
classdef PlotGraph < handle

  properties (Access = private)
    fig
    legendNames
  end
  
  methods (Access = public)
    
    function this = PlotGraph()
      this.fig = figure();
      set(this.fig, 'Visible', 'off');
      set(this.fig, 'PaperType', 'A4');
      set(this.fig, 'PaperOrientation', 'landscape');
      set(this.fig, 'PaperUnits', 'normalized');
      set(this.fig, 'PaperPosition', [0 0 1 1]);
      
      this.legendNames = {};
    end
    
    %
    % グラフのタイトルを設定する。
    %   引数: title 関数と同じ
    %
    function Title(this, varargin)
      figure(this.fig);
      title(varargin{:});
    end
    
    %
    % グラフの X 軸のラベルを設定する。
    %   引数: xlabel 関数と同じ
    %
    function XLabel(this, varargin)
      figure(this.fig);
      xlabel(varargin{:});
    end
    
    %
    % グラフの Y 軸のラベルを設定する。
    %   引数: ylabel 関数と同じ
    %
    function YLabel(this, varargin)
      figure(this.fig);
      ylabel(varargin{:});
    end
    
    %
    % グラフに系列を追加する。
    %   legendName: 凡例表示名
    %   x: X 値の配列
    %   y: Y 値の配列
    %   varargin: 線種に関する追加の引数
    %
    function AddSeries(this, legendName, x, y, varargin)
      figure(this.fig);
      hold all;
      if nargin >= 5
        plot(x, y, varargin{:});
      else
        plot(x, y);
      end
      this.legendNames = horzcat(this.legendNames, legendName);
    end
    
    %
    % グラフを PDF ファイルに保存する。
    %   filename: 保存する PDF ファイル名
    %
    function SaveAs(this, filename)
      figure(this.fig);
      legend(this.legendNames{:});
      print(this.fig, '-dpdf', filename);
    end
    
  end
  
  methods (Access = private)
  
    function delete(this)
      close(this.fig);
    end
    
  end
  
end
