%%% Monte Carlo Dynamic Classifier tool - MCDC tool

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
% Floor, Boston, MA 02110-1301 USA.

%
% Kitagawa system
%
% a1 = 0.5, b1 = 28, c1 =  8
% a2 = 0.6, b2 = 30, c2 = 10
% d1 = 0.05, d2 = 0.06, d3 = 0.07, d4 = 0.08

function [x, y] = KitagawaModel(T, a1, b1, c1, a2, b2, c2, d1, d2, d3, d4, q, r)

  q1 = q;
  q2 = q;
  r1 = r;
  r2 = r;
  r3 = r;

  x1 = zeros(1, T);
  x2 = zeros(1, T);
  y1 = zeros(1, T);
  y2 = zeros(1, T);
  y3 = zeros(1, T);

  x1(1) = 0;
  x2(1) = 0;

  for t = 1:T

    % Latent variables
    if t ~= 1
      x1(t) = a1 * x1(t-1) + b1 * x1(t-1) / (1 + x1(t-1) ^ 2) + c1 * cos(1.2 * t) + sqrt(q1) * randn;
      x2(t) = a2 * x2(t-1) + b2 * x2(t-1) / (1 + x2(t-1) ^ 2) + c2 * cos(1.2 * t) + sqrt(q2) * randn;
    end

    % Observations
    y1(t) = d1 * x1(t) ^ 2 + d2 * x2(t) ^ 2 + sqrt(r1) * randn;
    y2(t) = d3 * x1(t) ^ 2                  + sqrt(r2) * randn;
    y3(t) =                  d4 * x2(t)     + sqrt(r3) * randn;

  end

  x = [x1' x2'];
  y = [y1' y2' y3'];

end
