%%% Monte Carlo Dynamic Classifier tool - MCDC tool

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
% Floor, Boston, MA 02110-1301 USA.

%
% LorenzModelGPPFEstimation.m
%

rng('shuffle');

%
% Generate state and observation time series from Lorenz model
%

T      = 1000;
Sigma  = 10;
Rho    = 28;
Beta   = 8/3;
Alpha0 = 1;
q      = 1e-1;
r      = 1e-1;
dt     = 1e-2;

[x, y] = LorenzModel(T, Sigma, Rho, Beta, Alpha0, q, r, dt);

save('logs/LorenzModelGPPF_Data.mat');

grids = { ...
  [-30:2:30], ...
  [-30:2:30]  ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

obsKernelGens = {
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

stateMeanFuncs = { ...
  @(x) x(1,:), ...
  @(x) x(2,:)  ...
};

obsMeanFuncs = { ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2)))  ...
};

x0 = zeros(1, 2);
N  = 500;
J  = 2000;

aspect = Aspect();
aspect.LogFileName = 'logs/LorenzModelGPPF.log';
aspect.MatFileNamePrefix = 'logs/LorenzModelGPPF';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 100;

disp('GPPF Estimation using SPAPI');

splineHandle = @GenericSpline;
[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  GPPF(), ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  0, ... % Not used in GPPF estimation
  splineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  [], ... % No controls
  y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

save('logs/LorenzModelGPPF.mat');

disp('Done');
