%%% Monte Carlo Dynamic Classifier tool - MCDC tool

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
% Floor, Boston, MA 02110-1301 USA.

%
% KalmanFilter.m
%

% x(t) = F * x(t-1) + B * u(t-1) + w(t)   where w(t) ~ N(0, Q)
% y(t) = H * x(t)                + v(t)   where v(t) ~ N(0, R)
%
% x1 = initial state
% P1 = initial error covariance
% u  = controls
% z  = observations

function [x, y] = KalmanFilter(F, B, H, Q, R, x1, P1, u, z)

  xdim = size(x1, 1);
  ydim = size(z, 1);
  tmax = size(z, 2);

  x = nan(xdim, tmax);
  y = nan(ydim, tmax);
  x(:, 1) = x1;
  P = P1;

  for t = 2:tmax

    % Predict
    x(:, t) = F * x(:, t-1) + B * u(:, t-1);
    P = F * P * F' + Q;

    % Update
    y(:, t) = H * x(:, t);
    r(:, t) = z(:, t) - y(:, t);
    S = H * P * H' + R;
    K = P * H' * inv(S);
    x(:, t) = x(:, t) + K * r(:, t);
    P = (eye(xdim) - K * H) * P;

  end

end
